@@ -1,15 +1,14 @@
-from collections import Iterable
 from difflib import unified_diff
+from functools import partial
 from pprint import pformat
 from re import compile, MULTILINE
 from types import GeneratorType
 
 from testfixtures import not_there
-from testfixtures.compat import (
-    ClassType, Unicode, basestring, mock_call, unittest_mock_call,
-    PY3)
+from testfixtures.compat import ClassType, Iterable, Unicode, basestring, PY3
 from testfixtures.resolve import resolve
 from testfixtures.utils import indent
+from testfixtures.mock import parent_name, mock_call, unittest_mock_call
 
 
 def compare_simple(x, y, context):
@@ -17,37 +16,69 @@ def compare_simple(x, y, context):
     Returns a very simple textual difference between the two supplied objects.
     """
     if x != y:
-        return context.label('x', repr(x)) + ' != ' + context.label('y', repr(y))
+        repr_x = repr(x)
+        repr_y = repr(y)
+        if repr_x == repr_y:
+            x_attrs = _extract_attrs(x)
+            y_attrs = _extract_attrs(y)
+            diff = _compare_mapping(x_attrs, y_attrs, context, x,
+                                    'attributes ', '.%s')
+            if not diff:
+                return 'Both %s and %s appear as %r, but are not equal!' % (
+                    context.x_label or 'x', context.y_label or 'y', repr_x
+                )
+            return diff
+        return context.label('x', repr_x) + ' != ' + context.label('y', repr_y)
 
 
-def _extract_attrs(obj):
-    slots = getattr(obj, '__slots__', None)
-    if slots:
-        attrs = {}
+def _extract_attrs(obj, ignore=None):
+    try:
+        attrs = vars(obj).copy()
+    except TypeError:
+        attrs = None
+    else:
+        if isinstance(obj, BaseException):
+            attrs['args'] = obj.args
+
+    has_slots = getattr(obj, '__slots__', not_there) is not not_there
+    if has_slots:
+        slots = set()
+        for cls in type(obj).__mro__:
+            slots.update(getattr(cls, '__slots__', ()))
+        if slots and attrs is None:
+            attrs = {}
         for n in slots:
             value = getattr(obj, n, not_there)
             if value is not not_there:
                 attrs[n] = value
-    else:
-        try:
-            attrs = vars(obj).copy()
-        except TypeError:
-            return None
-        else:
-            if isinstance(obj, BaseException):
-                attrs['args'] = obj.args
+
+    if attrs is None:
+        return None
+
+    if ignore is not None:
+        if isinstance(ignore, dict):
+            ignore = ignore.get(type(obj), ())
+        for attr in ignore:
+            attrs.pop(attr, None)
     return attrs
 
 
 def compare_object(x, y, context):
     """
     Compare the two supplied objects based on their type and attributes.
+
+    :param ignore_attributes:
+
+       Either a sequence of strings containing attribute names to be ignored
+       when comparing or a mapping of type to sequence of strings containing
+       attribute names to be ignored when comparing that type.
     """
+    ignore_attributes = context.get_option('ignore_attributes', ())
     if type(x) is not type(y) or isinstance(x, ClassType):
         return compare_simple(x, y, context)
-    x_attrs = _extract_attrs(x)
-    y_attrs = _extract_attrs(y)
-    if x_attrs is None or y_attrs is None:
+    x_attrs = _extract_attrs(x, ignore_attributes)
+    y_attrs = _extract_attrs(y, ignore_attributes)
+    if x_attrs is None or y_attrs is None or not (x_attrs and y_attrs):
         return compare_simple(x, y, context)
     if x_attrs != y_attrs:
         return _compare_mapping(x_attrs, y_attrs, context, x,
@@ -318,10 +349,18 @@ def compare_call(x, y, context):
     x_name, x_args, x_kw = x
     y_name, y_args, y_kw = y
     if x_name == y_name and x_args == y_args and x_kw == y_kw:
-        return compare_call(x.parent, y.parent, context)
+        return compare_call(getattr(x, parent_name), getattr(y, parent_name), context)
     return compare_text(repr(x), repr(y), context)
 
 
+def compare_partial(x, y, context):
+    x_attrs = dict(func=x.func, args=x.args, keywords=x.keywords)
+    y_attrs = dict(func=y.func, args=y.args, keywords=y.keywords)
+    if x_attrs != y_attrs:
+        return _compare_mapping(x_attrs, y_attrs, context, x,
+                                'attributes ', '.%s')
+
+
 def _short_repr(obj):
     repr_ = repr(obj)
     if len(repr_) > 30:
@@ -342,6 +381,7 @@ _registry = {
     mock_call.__class__: compare_call,
     unittest_mock_call.__class__: compare_call,
     BaseException: compare_exception,
+    partial: compare_partial,
     }
 
 if PY3:
@@ -461,6 +501,9 @@ class CompareContext(object):
         return '\n\nWhile comparing %s: ' % ''.join(self.breadcrumbs[1:])
 
     def seen(self, x, y):
+        # don't get confused by string interning:
+        if isinstance(x, basestring) and isinstance(y, basestring):
+            return False
         key = id(x), id(y)
         if key in self._seen:
             return True
