@@ -6,7 +6,7 @@ from collections import namedtuple
 
 from testfixtures.shouldraise import ShouldAssert
 from testfixtures.tests.sample1 import SampleClassA, SampleClassB, Slotted
-from .mock import Mock, call
+from testfixtures.mock import Mock, call
 from re import compile
 from testfixtures import (
     Comparison as C,
@@ -19,7 +19,7 @@ from testfixtures import (
 from testfixtures.compat import (
     class_type_name, exception_module, PY3, xrange,
     BytesLiteral, UnicodeLiteral,
-    PY2
+    PY2, PY_37_PLUS, ABC
 )
 from testfixtures.comparison import compare_sequence
 from unittest import TestCase
@@ -34,6 +34,7 @@ call_list_repr = repr(Mock().mock_calls.__class__)
 
 marker = object()
 
+_compare = compare
 
 class CompareHelper(object):
 
@@ -51,8 +52,8 @@ class CompareHelper(object):
             actual = hexsub(e.args[0])
             if message is not None:
                 # handy for debugging, but can't be relied on for tests!
-                compare(actual, expected=message, show_whitespace=True)
-                assert actual==message
+                _compare(actual, expected=message, show_whitespace=True)
+                assert actual == message
             else:
                 if not regex.match(actual):  # pragma: no cover
                     raise AssertionError(
@@ -202,10 +203,16 @@ class TestCompare(CompareHelper, TestCase):
     def test_exception_diff(self):
         e1 = ValueError('some message')
         e2 = ValueError('some other message')
-        self.check_raises(
-            e1, e2,
-            "ValueError('some message',) != ValueError('some other message',)"
-            )
+        if PY_37_PLUS:
+            self.check_raises(
+                e1, e2,
+                "ValueError('some message') != ValueError('some other message')"
+                )
+        else:
+            self.check_raises(
+                e1, e2,
+                "ValueError('some message',) != ValueError('some other message',)"
+                )
 
     def test_exception_diff_c_wrapper(self):
         e1 = ValueError('some message')
@@ -213,13 +220,14 @@ class TestCompare(CompareHelper, TestCase):
         self.check_raises(
             C(e1), e2,
             ("\n"
-             "<C(failed):{0}.ValueError>\n"
+             "<C(failed):{}.ValueError>\n"
              "attributes differ:\n"
              "'args': ('some message',) (Comparison) "
              "!= ('some other message',) (actual)\n"
              "</C>"
-             " != ValueError('some other message',)"
-             ).format(exception_module))
+             " != ValueError('some other message'{})"
+             ).format(exception_module,
+                      '' if PY_37_PLUS else ','))
 
     def test_sequence_long(self):
         self.check_raises(
@@ -1339,7 +1347,7 @@ b
 
     def test_invalid_zero_args(self):
         with ShouldRaise(TypeError(
-                'Exactly two objects needed, you supplied: []'
+                'Exactly two objects needed, you supplied:'
         )):
             compare()
 
@@ -1355,6 +1363,12 @@ b
         )):
             compare('x', 'y', 'z')
 
+    def test_invalid_because_of_typo(self):
+        with ShouldRaise(TypeError(
+                "Exactly two objects needed, you supplied: ['x'] {'expceted': 'z'}"
+        )):
+            compare('x', expceted='z')
+
     def test_dont_raise(self):
         self.assertEqual(compare('x', 'y', raises=False), "'x' != 'y'")
 
@@ -1452,29 +1466,11 @@ b
             "second:\n"
             "[call.bar(1, 3, x=7, y=4)]"
             "\n\n"
-            'While comparing [0]: mock.call not as expected:\n'
-            '\n'
-            "While comparing [0] function name: 'foo' != 'bar'\n"
-            '\n'
-            'While comparing [0] args: sequence not as expected:\n'
-            '\n'
-            'same:\n'
-            '(1,)\n'
-            '\n'
-            'first:\n'
-            '(2,)\n'
-            '\n'
-            'second:\n'
-            '(3,)\n'
-            '\n'
-            'While comparing [0] kw: dict not as expected:\n'
-            '\n'
-            'same:\n'
-            "['y']\n"
-            '\n'
-            'values differ:\n'
-            "'x': 3 != 7"
-            )
+            'While comparing [0]: \n'
+            "'call.foo(1, 2, x=3, y=4)'\n"
+            '!=\n'
+            "'call.bar(1, 3, x=7, y=4)'"
+        )
 
     def test_compare_arbitrary_nested_same(self):
         compare(SampleClassA([SampleClassB()]),
@@ -1586,3 +1582,267 @@ b
             "datetime.datetime(2001, 1, 1, 0, 0) != "
             "datetime.datetime(2001, 1, 2, 0, 0)"
         )
+
+    def test_inherited_slots(self):
+
+        class Parent(object):
+            __slots__ = ('a',)
+
+        class Child(Parent):
+            __slots__ = ('b',)
+
+            def __init__(self, a, b):
+                self.a, self.b = a, b
+
+        self.check_raises(
+            Child(1, 'x'),
+            Child(2, 'x'),
+            'Child not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['b']\n"
+            '\n'
+            'attributes differ:\n'
+            "'a': 1 != 2"
+        )
+
+    def test_empty_child_slots(self):
+
+        class Parent(object):
+            __slots__ = ('a',)
+
+            def __init__(self, a):
+                self.a = a
+
+        class Child(Parent):
+            __slots__ = ()
+
+        compare(Child(1), Child(1))
+
+    def test_slots_and_attrs(self):
+
+        class Parent(object):
+            __slots__ = ('a',)
+
+        class Child(Parent):
+            def __init__(self, a, b):
+                self.a = a
+                self.b = b
+
+        self.check_raises(Child(1, 2), Child(1, 3), message=(
+            'Child not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['a']\n"
+            '\n'
+            'attributes differ:\n'
+            "'b': 2 != 3"
+        ))
+
+    def test_partial_callable_different(self):
+
+        def foo(x): pass
+        def bar(y): pass
+
+        self.check_raises(
+            partial(foo),
+            partial(bar),
+            (
+                'partial not as expected:\n'
+                '\n'
+                'attributes same:\n'
+                "['args', 'keywords']\n"
+                '\n'
+                'attributes differ:\n'
+                "'func': {foo} != {bar}\n"
+                '\n'
+                'While comparing .func: {foo} != {bar}'
+            ).format(foo=hexsub(repr(foo)), bar=hexsub(repr(bar))))
+
+    def test_partial_args_different(self):
+
+        def foo(x): pass
+
+        self.check_raises(
+            partial(foo, 1),
+            partial(foo, 2),
+            'partial not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['func', 'keywords']\n"
+            '\n'
+            'attributes differ:\n'
+            "'args': (1,) != (2,)\n"
+            '\n'
+            'While comparing .args: sequence not as expected:\n'
+            '\n'
+            'same:\n'
+            '()\n'
+            '\n'
+            'first:\n'
+            '(1,)\n'
+            '\n'
+            'second:\n'
+            '(2,)'
+        )
+
+    def test_partial_kw_different(self):
+
+        def foo(x): pass
+
+        self.check_raises(
+            partial(foo, x=1, y=3),
+            partial(foo, x=2, z=4),
+            'partial not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['args', 'func']\n"
+            '\n'
+            'attributes differ:\n'
+            "'keywords': {'x': 1, 'y': 3} != {'x': 2, 'z': 4}\n"
+            '\n'
+            'While comparing .keywords: dict not as expected:\n'
+            '\n'
+            'in first but not second:\n'
+            "'y': 3\n"
+            '\n'
+            'in second but not first:\n'
+            "'z': 4\n"
+            '\n'
+            'values differ:\n'
+            "'x': 1 != 2"
+        )
+
+    def test_partial_equal(self):
+
+        def foo(x): pass
+
+        compare(partial(foo, 1, x=2), partial(foo, 1, x=2))
+
+    def test_repr_and_attributes_equal(self):
+
+        class Wut(object):
+            def __repr__(self):
+                return 'Wut'
+            def __eq__(self, other):
+                return False
+
+        self.check_raises(
+            Wut(),
+            Wut(),
+            "Both x and y appear as 'Wut', but are not equal!"
+        )
+
+        self.check_raises(
+            expected=Wut(),
+            actual=Wut(),
+            message="Both expected and actual appear as 'Wut', but are not equal!"
+        )
+
+    def test_not_recursive(self):
+        self.check_raises(
+            {1: 'foo', 2: 'foo'},
+            {1: 'bar', 2: 'bar'},
+            "dict not as expected:\n"
+            "\n"
+            "values differ:\n"
+            "1: 'foo' != 'bar'\n"
+            "2: 'foo' != 'bar'\n"
+            "\n"
+            "While comparing [1]: 'foo' != 'bar'"
+            "\n\n"
+            "While comparing [2]: 'foo' != 'bar'"
+            )
+
+
+class TestIgnore(CompareHelper):
+
+    class Parent(object):
+        def __init__(self, id, other):
+            self.id = id
+            self.other = other
+        def __repr__(self):
+            return '<{}:{}>'.format(type(self).__name__, self.id)
+
+    class Child(Parent): pass
+
+    def test_ignore_attributes(self):
+        compare(self.Parent(1, 3), self.Parent(2, 3), ignore_attributes={'id'})
+
+    def test_ignore_attributes_different_types(self):
+        self.check_raises(
+            self.Parent(1, 3),
+            self.Child(2, 3),
+            '<Parent:1> != <Child:2>',
+            ignore_attributes={'id'}
+        )
+
+    def test_ignore_attributes_per_type(self):
+        ignore = {self.Parent: {'id'}}
+        compare(self.Parent(1, 3), self.Parent(2, 3), ignore_attributes=ignore)
+        self.check_raises(
+            self.Child(1, 3),
+            self.Child(2, 3),
+            'Child not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['other']\n"
+            '\n'
+            'attributes differ:\n'
+            "'id': 1 != 2",
+            ignore_attributes=ignore
+        )
+
+
+class BaseClass(ABC):
+    pass
+
+
+class MyDerivedClass(BaseClass):
+
+    def __init__(self, thing):
+        self.thing = thing
+
+
+class ConcreteBaseClass(object): pass
+
+
+class ConcreteDerivedClass(ConcreteBaseClass):
+
+    def __init__(self, thing):
+        self.thing = thing
+
+
+class TestBaseClasses(CompareHelper):
+
+    def test_abc_equal(self):
+        thing1 = MyDerivedClass(1)
+        thing2 = MyDerivedClass(1)
+
+        compare(thing1, thing2)
+
+    def test_abc_unequal(self):
+        thing1 = MyDerivedClass(1)
+        thing2 = MyDerivedClass(2)
+
+        self.check_raises(thing1, thing2, message=(
+            "MyDerivedClass not as expected:\n\n"
+            "attributes differ:\n"
+            "'thing': 1 != 2"
+        ))
+
+    def test_concrete_equal(self):
+        thing1 = ConcreteDerivedClass(1)
+        thing2 = ConcreteDerivedClass(1)
+
+        compare(thing1, thing2)
+
+    def test_concrete_unequal(self):
+        thing1 = ConcreteDerivedClass(1)
+        thing2 = ConcreteDerivedClass(2)
+
+        self.check_raises(thing1, thing2, message=(
+            "ConcreteDerivedClass not as expected:\n\n"
+            "attributes differ:\n"
+            "'thing': 1 != 2"
+        ))
